package com.sitemesh.filter;

import com.sitemesh.Page;
import com.sitemesh.PageParser;
import com.sitemesh.Factory;
import java.io.IOException;
import javax.servlet.Config;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

/**
 * Implementation of HttpServletResponse that captures page data instead of
 * sending to output stream.
 *
 * <p>Should be used in filter-chains or when forwarding/including pages
 * using a RequestDispatcher.</p>
 *
 * <p>In order to capture the response, {@link #getOutputStream()} returns
 * an instance of {@link com.sitemesh.filter.PageOutputStream} .</p>
 *
 * @author <a href="joe@truemesh.com">Joe Walnes</a>
 * @version $Revision: 1.8 $
 */
public class PageResponse extends HttpServletResponseWrapper {

	/**
	 * @label returns
	 */
	private PageOutputStream pageOutputStream;

	private Page page;
	private String contentType;
	private Config config;

	/**
	 * Create PageResponse wrapped around existing HttpServletResponse.
	 */
	public PageResponse( HttpServletResponse response, Config config ) throws IOException {
		super( response );
		pageOutputStream = new PageOutputStream();
		this.config = config;
	}

	/**
	 * Set the content-type of the request and store it so it can
	 * be passed to the PageParser.
	 */
	public void setContentType( String contentType ) {
		this.contentType = contentType;
		super.setContentType( contentType );
	}

	/**
	 * Return instance of {@link com.sitemesh.filter.PageOutputStream}
	 * allowing all data written to stream to be stored in temporary
	 * buffer.
	 */
	public ServletOutputStream getOutputStream() throws IOException {
		return pageOutputStream;
	}

	/**
	 * Flush and close output stream of wrapped response.
	 */
	public void closeStream() throws IOException {
		super.getOutputStream().flush();
		super.getOutputStream().close();
	}

	/**
	 * Send data written to PageOutputStream to PageParser and return
	 * Page instance.
	 */
	public Page getPage() throws IOException {
		if ( page == null ) {
			Factory factory   = Factory.getInstance( config );
			PageParser parser = factory.getPageParser( contentType );
			return parser.parse( pageOutputStream.getBytes() );
		}
		return page;
	}

}
